module Pasta
  module Ingredients     

    def self.included(base)
      base.extend ClassMethods
    end
       
    # Public. Returns the grammars of a recipe
    # 
    # name - the name of a grammar rule as a Symbol
    #
    # Example
    #   grammars(:html)
    #   # => <Proc:>
    #
    # Returns a Proc containing grammar rules
    def grammars(name=nil)
      return name.nil? ? self.class.grammars : self.class.grammars[name]
    end 

    module ClassMethods    
      @@grammars = {}
      
      def grammars
        @@grammars
      end
         
      # Public. The DSL for declaring grammar rules in a recipe.
      #
      # Recipes contain grammars that are defined in ordered arrays. Grammars are recursively matched in a top down
      #   fashion where the array elements match ingredients.  Grammar matching is done on a first match basis so that
      #   the last rule can be a catch-all one.
      #
      # name  - a Symbol naming the grammar
      # rules - a block defining the grammar
      #
      # Example
      #   grammar(:html)       { [:blockquotes, :paragraphs] }
      #   grammar(:blockquote) { [:blockquote, :paragraphs] }
      #   grammar(:paragraph)  { [:links, :emphasis, :bold, :text] }
      #
      # Returns nothing
      def grammar(name, &rules)
        rules.call.each { |rule| require "pasta/ingredients/#{rule.to_s}" }
        @@grammars[name] = rules.call
      end
    end
  end
end