module Pasta
  module Ingredients
    
    # Parses both links and images
    # $1 => "!", $2 => link text, $4 => inline url, $5 => reference id
    LINKS = /([!]*)\[(.+?)\](\((.+?)\)|\s?\[(.*?)\])/
    
    LINK_DEFINITION = /^ {0,3}\[([\w\s]+?)\]: (.+?)(^\s*[\n\r]|\z)/m
    
    def links(text, html)
      find_link_definitions(text)
      
      text.gsub!(LINKS) do |match|
        link_text, target = $2, ''
        if $4
          # It's an inline link
          target = $4
        else
          # It's a reference link
          id = $5 == '' ? $2.downcase : $5.downcase
          break unless @link_definitions && @link_definitions.key?(id)
          target = @link_definitions[id][:target]
        end
        # format image or link
        url, title = parse_link target
        if $1 == '!'
          match = "<img src=\"#{url}\" alt=\"#{link_text}\"#{title} />"
        else
          match = "<a href=\"#{url}\"#{title}>#{link_text}</a>"
        end
      end
      
      text = clean_link_definitions(text)
      [text, html]
    end
    
    def find_link_definitions(text)
      @link_definitions = {}
      text.gsub(LINK_DEFINITION) do |match|
        @link_definitions[$1.downcase] = {target: $2, used: false}
      end
    end 
    
    def clean_link_definitions(text)
      @link_definitions.each do |key,val|
        text.gsub!(/^ {0,3}\[#{key}\]: (.+?)(^\s*[\n\r]|\z)/mi, '')
      end
      text
    end
    
    def parse_link(text)
      target = text.split(' ')
      url = target[0].gsub(/[<>]/, '')
      title = " title=#{target[1..-1].join(' ').strip}" if target.length > 1
      [url, title]
    end
  end
end